@@ -25,69 +25,65 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <cmake.h>
-#include <TDB2.h>
-#include <iostream>
-#include <sstream>
-#include <algorithm>
-#include <vector>
-#include <list>
-#include <unordered_set>
-#include <stdlib.h>
-#include <signal.h>
-#include <Context.h>
+// cmake.h include header must come first
+
 #include <Color.h>
+#include <Context.h>
 #include <Datetime.h>
+#include <TDB2.h>
 #include <Table.h>
-#include <shared.h>
 #include <format.h>
 #include <main.h>
+#include <shared.h>
+#include <signal.h>
+#include <stdlib.h>
 #include <util.h>
+
+#include <algorithm>
+#include <iostream>
+#include <list>
+#include <sstream>
+#include <unordered_set>
+#include <vector>
+
 #include "tc/Server.h"
 #include "tc/util.h"
 
 bool TDB2::debug_mode = false;
-static void dependency_scan (std::vector<Task> &);
+static void dependency_scan(std::vector<Task>&);
 
 ////////////////////////////////////////////////////////////////////////////////
-TDB2::TDB2 ()
-: replica {tc::Replica()} // in-memory Replica
-, _working_set {}
-{
-}
+TDB2::TDB2()
+    : replica{tc::Replica()}  // in-memory Replica
+      ,
+      _working_set{} {}
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::open_replica (const std::string& location, bool create_if_missing)
-{
-  File pending_data = File (location + "/pending.data");
-  if (pending_data.exists()) {
-    Color warning = Color (Context::getContext ().config.get ("color.warning"));
-    std::cerr << warning.colorize (
-      format ("Found existing '.data' files in {1}", location)) << "\n";
-    std::cerr << "  Taskwarrior's storage format changed in 3.0, requiring a manual migration.\n";
-    std::cerr << "  See https://github.com/GothenburgBitFactory/taskwarrior/releases.\n";
-  }
+void TDB2::open_replica(const std::string& location, bool create_if_missing) {
   replica = tc::Replica(location, create_if_missing);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Add the new task to the replica.
-void TDB2::add (Task& task)
-{
+void TDB2::add(Task& task) {
   // Ensure the task is consistent, and provide defaults if necessary.
   // bool argument to validate() is "applyDefault", to apply default values for
   // properties not otherwise given.
-  task.validate (true);
+  task.validate(true);
 
-  std::string uuid = task.get ("uuid");
+  std::string uuid = task.get("uuid");
   changes[uuid] = task;
 
-  auto innertask = replica.import_task_with_uuid (uuid);
+  // run hooks for this new task
+  Context::getContext().hooks.onAdd(task);
+
+  auto innertask = replica.import_task_with_uuid(uuid);
 
   {
     auto guard = replica.mutate_task(innertask);
 
     // add the task attributes
-    for (auto& attr : task.all ()) {
+    for (auto& attr : task.all()) {
       // TaskChampion does not store uuid or id in the taskmap
       if (attr == "uuid" || attr == "id") {
         continue;
@@ -96,38 +92,34 @@ void TDB2::add (Task& task)
       // Use `set_status` for the task status, to get expected behavior
       // with respect to the working set.
       else if (attr == "status") {
-        innertask.set_status (Task::status2tc (Task::textToStatus (task.get (attr))));
+        innertask.set_status(Task::status2tc(Task::textToStatus(task.get(attr))));
       }
 
       // use `set_modified` to set the modified timestamp, avoiding automatic
       // updates to this field by TaskChampion.
       else if (attr == "modified") {
-        auto mod = (time_t) std::stoi (task.get (attr));
-        innertask.set_modified (mod);
+        auto mod = (time_t)std::stoi(task.get(attr));
+        innertask.set_modified(mod);
       }
 
       // otherwise, just set the k/v map value
       else {
-        innertask.set_value (attr, std::make_optional (task.get (attr)));
+        innertask.set_value(attr, std::make_optional(task.get(attr)));
       }
     }
   }
 
-  auto ws = replica.working_set ();
+  auto ws = replica.working_set();
 
   // get the ID that was assigned to this task
-  auto id = ws.by_uuid (uuid);
+  auto id = ws.by_uuid(uuid);
 
   // update the cached working set with the new information
-  _working_set = std::make_optional (std::move (ws));
+  _working_set = std::make_optional(std::move(ws));
 
-  // run hooks for this new task
-  Context::getContext ().hooks.onAdd (task);
-
-  if (id.has_value ()) {
-      task.id = id.value();
+  if (id.has_value()) {
+    task.id = id.value();
   }
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -145,31 +137,30 @@ void TDB2::add (Task& task)
 // this method. In this case, this method throws an error that will make sense
 // to the user. This is especially unlikely since tasks are only deleted when
 // they have been unmodified for a long time.
-void TDB2::modify (Task& task)
-{
+void TDB2::modify(Task& task) {
   // All locally modified tasks are timestamped, implicitly overwriting any
   // changes the user or hooks tried to apply to the "modified" attribute.
-  task.setAsNow ("modified");
-  task.validate (false);
-  auto uuid = task.get ("uuid");
+  task.setAsNow("modified");
+  task.validate(false);
+  auto uuid = task.get("uuid");
 
   changes[uuid] = task;
 
-	// invoke the hook and allow it to modify the task before updating
+  // invoke the hook and allow it to modify the task before updating
   Task original;
-  get (uuid, original);
-	Context::getContext ().hooks.onModify (original, task);
+  get(uuid, original);
+  Context::getContext().hooks.onModify(original, task);
 
-  auto maybe_tctask = replica.get_task (uuid);
-  if (!maybe_tctask.has_value ()) {
-    throw std::string ("task no longer exists");
+  auto maybe_tctask = replica.get_task(uuid);
+  if (!maybe_tctask.has_value()) {
+    throw std::string("task no longer exists");
   }
-  auto tctask = std::move (maybe_tctask.value ());
+  auto tctask = std::move(maybe_tctask.value());
   auto guard = replica.mutate_task(tctask);
-  auto tctask_map = tctask.get_taskmap ();
+  auto tctask_map = tctask.get_taskmap();
 
   std::unordered_set<std::string> seen;
-  for (auto k : task.all ()) {
+  for (auto k : task.all()) {
     // ignore task keys that aren't stored
     if (k == "uuid") {
       continue;
@@ -189,31 +180,35 @@ void TDB2::modify (Task& task)
       if (v_new == "") {
         tctask.set_value(k, {});
       } else {
-        tctask.set_value(k, make_optional (v_new));
+        tctask.set_value(k, make_optional(v_new));
       }
     }
   }
 
   // we've now added and updated properties; but must find any deleted properties
   for (auto kv : tctask_map) {
-    if (seen.find (kv.first) == seen.end ()) {
-      tctask.set_value (kv.first, {});
+    if (seen.find(kv.first) == seen.end()) {
+      tctask.set_value(kv.first, {});
     }
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const tc::WorkingSet &TDB2::working_set ()
-{
-  if (!_working_set.has_value ()) {
-    _working_set = std::make_optional (replica.working_set ());
-  }
-  return _working_set.value ();
+void TDB2::purge(Task& task) {
+  auto uuid = task.get("uuid");
+  replica.delete_task(uuid);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::get_changes (std::vector <Task>& changes)
-{
+const tc::WorkingSet& TDB2::working_set() {
+  if (!_working_set.has_value()) {
+    _working_set = std::make_optional(replica.working_set());
+  }
+  return _working_set.value();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void TDB2::get_changes(std::vector<Task>& changes) {
   std::map<std::string, Task>& changes_map = this->changes;
   changes.clear();
   std::transform(changes_map.begin(), changes_map.end(), std::back_inserter(changes),
@@ -221,8 +216,7 @@ void TDB2::get_changes (std::vector <Task>& changes)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::revert ()
-{
+void TDB2::revert() {
   auto undo_ops = replica.get_undo_ops();
   if (undo_ops.len == 0) {
     std::cout << "No operations to undo.";
@@ -234,19 +228,18 @@ void TDB2::revert ()
   } else {
     replica.free_replica_ops(undo_ops);
   }
-  replica.rebuild_working_set (false);
+  replica.rebuild_working_set(false);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool TDB2::confirm_revert (struct tc::ffi::TCReplicaOpList undo_ops)
-{
+bool TDB2::confirm_revert(struct tc::ffi::TCReplicaOpList undo_ops) {
   // TODO Use show_diff rather than this basic listing of operations, though
   // this might be a worthy undo.style itself.
   std::cout << "The following " << undo_ops.len << " operations would be reverted:\n";
   for (size_t i = 0; i < undo_ops.len; i++) {
     std::cout << "- ";
     tc::ffi::TCReplicaOp op = undo_ops.items[i];
-    switch(op.operation_type) {
+    switch (op.operation_type) {
       case tc::ffi::TCReplicaOpType::Create:
         std::cout << "Create " << replica.get_op_uuid(op);
         break;
@@ -255,107 +248,100 @@ bool TDB2::confirm_revert (struct tc::ffi::TCReplicaOpList undo_ops)
         break;
       case tc::ffi::TCReplicaOpType::Update:
         std::cout << "Update " << replica.get_op_uuid(op) << "\n";
-        std::cout << "    " << replica.get_op_property(op) << ": " << option_string(replica.get_op_old_value(op)) << " -> " << option_string(replica.get_op_value(op));
+        std::cout << "    " << replica.get_op_property(op) << ": "
+                  << option_string(replica.get_op_old_value(op)) << " -> "
+                  << option_string(replica.get_op_value(op));
         break;
       case tc::ffi::TCReplicaOpType::UndoPoint:
-        throw std::string ("Can't undo UndoPoint.");
+        throw std::string("Can't undo UndoPoint.");
         break;
       default:
-        throw std::string ("Can't undo non-operation.");
+        throw std::string("Can't undo non-operation.");
         break;
     }
     std::cout << "\n";
   }
-  return ! Context::getContext ().config.getBoolean ("confirmation") ||
-        confirm ("The undo command is not reversible.  Are you sure you want to revert to the previous state?");
+  return !Context::getContext().config.getBoolean("confirmation") ||
+         confirm(
+             "The undo command is not reversible.  Are you sure you want to revert to the previous "
+             "state?");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string TDB2::option_string(std::string input) {
-  return input == "" ? "<empty>" : input;
-}
+std::string TDB2::option_string(std::string input) { return input == "" ? "<empty>" : input; }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::show_diff (
-  const std::string& current,
-  const std::string& prior,
-  const std::string& when)
-{
-  Datetime lastChange (strtoll (when.c_str (), nullptr, 10));
+void TDB2::show_diff(const std::string& current, const std::string& prior,
+                     const std::string& when) {
+  Datetime lastChange(strtoll(when.c_str(), nullptr, 10));
 
   // Set the colors.
-  Color color_red   (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.before") : "");
-  Color color_green (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.after") : "");
+  Color color_red(
+      Context::getContext().color() ? Context::getContext().config.get("color.undo.before") : "");
+  Color color_green(
+      Context::getContext().color() ? Context::getContext().config.get("color.undo.after") : "");
 
   auto before = prior == "" ? Task() : Task(prior);
   auto after = Task(current);
 
-  if (Context::getContext ().config.get ("undo.style") == "side")
-  {
+  if (Context::getContext().config.get("undo.style") == "side") {
     Table view = before.diffForUndoSide(after);
 
     std::cout << '\n'
-              << format ("The last modification was made {1}", lastChange.toString ())
+              << format("The last modification was made {1}", lastChange.toString()) << '\n'
               << '\n'
-              << '\n'
-              << view.render ()
-              << '\n';
+              << view.render() << '\n';
   }
 
-  else if (Context::getContext ().config.get ("undo.style") == "diff")
-  {
+  else if (Context::getContext().config.get("undo.style") == "diff") {
     Table view = before.diffForUndoPatch(after, lastChange);
-    std::cout << '\n'
-              << view.render ()
-              << '\n';
+    std::cout << '\n' << view.render() << '\n';
   }
 }
 
-void TDB2::gc ()
-{
+////////////////////////////////////////////////////////////////////////////////
+void TDB2::gc() {
   Timer timer;
 
   // Allowed as an override, but not recommended.
-  if (Context::getContext ().config.getBoolean ("gc"))
-  {
-    replica.rebuild_working_set (true);
+  if (Context::getContext().config.getBoolean("gc")) {
+    replica.rebuild_working_set(true);
   }
 
-  Context::getContext ().time_gc_us += timer.total_us ();
+  Context::getContext().time_gc_us += timer.total_us();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+void TDB2::expire_tasks() { replica.expire_tasks(); }
+
 ////////////////////////////////////////////////////////////////////////////////
 // Latest ID is that of the last pending task.
-int TDB2::latest_id ()
-{
-  const tc::WorkingSet &ws = working_set ();
-  return (int)ws.largest_index ();
+int TDB2::latest_id() {
+  const tc::WorkingSet& ws = working_set();
+  return (int)ws.largest_index();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::vector <Task> TDB2::all_tasks ()
-{
+const std::vector<Task> TDB2::all_tasks() {
   auto all_tctasks = replica.all_tasks();
-  std::vector <Task> all;
-  for (auto& tctask : all_tctasks)
-    all.push_back (Task (std::move (tctask)));
+  std::vector<Task> all;
+  for (auto& tctask : all_tctasks) all.push_back(Task(std::move(tctask)));
 
   return all;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::vector <Task> TDB2::pending_tasks ()
-{
-  const tc::WorkingSet &ws = working_set ();
-  auto largest_index = ws.largest_index ();
+const std::vector<Task> TDB2::pending_tasks() {
+  const tc::WorkingSet& ws = working_set();
+  auto largest_index = ws.largest_index();
 
-  std::vector <Task> result;
+  std::vector<Task> result;
   for (size_t i = 0; i <= largest_index; i++) {
-    auto maybe_uuid = ws.by_index (i);
-    if (maybe_uuid.has_value ()) {
-      auto maybe_task = replica.get_task (maybe_uuid.value ());
-      if (maybe_task.has_value ()) {
-        result.push_back (Task (std::move (maybe_task.value ())));
+    auto maybe_uuid = ws.by_index(i);
+    if (maybe_uuid.has_value()) {
+      auto maybe_task = replica.get_task(maybe_uuid.value());
+      if (maybe_task.has_value()) {
+        result.push_back(Task(std::move(maybe_task.value())));
       }
     }
   }
@@ -366,16 +352,15 @@ const std::vector <Task> TDB2::pending_tasks ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::vector <Task> TDB2::completed_tasks ()
-{
+const std::vector<Task> TDB2::completed_tasks() {
   auto all_tctasks = replica.all_tasks();
-  const tc::WorkingSet &ws = working_set ();
+  const tc::WorkingSet& ws = working_set();
 
-  std::vector <Task> result;
+  std::vector<Task> result;
   for (auto& tctask : all_tctasks) {
     // if this task is _not_ in the working set, return it.
-    if (!ws.by_uuid (tctask.get_uuid ())) {
-      result.push_back (Task (std::move (tctask)));
+    if (!ws.by_uuid(tctask.get_uuid())) {
+      result.push_back(Task(std::move(tctask)));
     }
   }
 
@@ -384,10 +369,9 @@ const std::vector <Task> TDB2::completed_tasks ()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Locate task by ID, wherever it is.
-bool TDB2::get (int id, Task& task)
-{
-  const tc::WorkingSet &ws = working_set ();
-  const auto maybe_uuid = ws.by_index (id);
+bool TDB2::get(int id, Task& task) {
+  const tc::WorkingSet& ws = working_set();
+  const auto maybe_uuid = ws.by_index(id);
   if (maybe_uuid) {
     auto maybe_task = replica.get_task(*maybe_uuid);
     if (maybe_task) {
@@ -401,25 +385,24 @@ bool TDB2::get (int id, Task& task)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Locate task by UUID, including by partial ID, wherever it is.
-bool TDB2::get (const std::string& uuid, Task& task)
-{
+bool TDB2::get(const std::string& uuid, Task& task) {
   // try by raw uuid, if the length is right
-  if (uuid.size () == 36) {
+  if (uuid.size() == 36) {
     try {
-      auto maybe_task = replica.get_task (uuid);
+      auto maybe_task = replica.get_task(uuid);
       if (maybe_task) {
-        task = Task{std::move (*maybe_task)};
+        task = Task{std::move(*maybe_task)};
         return true;
       }
-    } catch (const std::string &err) {
+    } catch (const std::string& err) {
       return false;
     }
   }
 
   // Nothing to do but iterate over all tasks and check whether it's closeEnough
-  for (auto& tctask : replica.all_tasks ()) {
-    if (closeEnough (tctask.get_uuid (), uuid, uuid.length ())) {
-      task = Task{std::move (tctask)};
+  for (auto& tctask : replica.all_tasks()) {
+    if (closeEnough(tctask.get_uuid(), uuid, uuid.length())) {
+      task = Task{std::move(tctask)};
       return true;
     }
   }
@@ -429,34 +412,25 @@ bool TDB2::get (const std::string& uuid, Task& task)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Locate task by UUID, wherever it is.
-bool TDB2::has (const std::string& uuid)
-{
+bool TDB2::has(const std::string& uuid) {
   Task task;
   return get(uuid, task);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::vector <Task> TDB2::siblings (Task& task)
-{
-  std::vector <Task> results;
-  if (task.has ("parent"))
-  {
-    std::string parent = task.get ("parent");
+const std::vector<Task> TDB2::siblings(Task& task) {
+  std::vector<Task> results;
+  if (task.has("parent")) {
+    std::string parent = task.get("parent");
 
-    for (auto& i : this->pending_tasks())
-    {
+    for (auto& i : this->pending_tasks()) {
       // Do not include self in results.
-      if (i.id != task.id)
-      {
+      if (i.id != task.id) {
         // Do not include completed or deleted tasks.
-        if (i.getStatus () != Task::completed &&
-            i.getStatus () != Task::deleted)
-        {
+        if (i.getStatus() != Task::completed && i.getStatus() != Task::deleted) {
           // If task has the same parent, it is a sibling.
-          if (i.has ("parent") &&
-              i.get ("parent") == parent)
-          {
-            results.push_back (i);
+          if (i.has("parent") && i.get("parent") == parent) {
+            results.push_back(i);
           }
         }
       }
@@ -467,41 +441,40 @@ const std::vector <Task> TDB2::siblings (Task& task)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::vector <Task> TDB2::children (Task& parent)
-{
+const std::vector<Task> TDB2::children(Task& parent) {
   // scan _pending_ tasks for those with `parent` equal to this task
-	std::vector <Task> results;
-  std::string this_uuid = parent.get ("uuid");
+  std::vector<Task> results;
+  std::string this_uuid = parent.get("uuid");
 
-  const tc::WorkingSet &ws = working_set ();
-  size_t end_idx = ws.largest_index ();
+  const tc::WorkingSet& ws = working_set();
+  size_t end_idx = ws.largest_index();
 
   for (size_t i = 0; i <= end_idx; i++) {
-    auto uuid_opt = ws.by_index (i);
+    auto uuid_opt = ws.by_index(i);
     if (!uuid_opt) {
       continue;
     }
-    auto uuid = uuid_opt.value ();
+    auto uuid = uuid_opt.value();
 
     // skip self-references
     if (uuid == this_uuid) {
       continue;
     }
 
-    auto task_opt = replica.get_task (uuid_opt.value ());
+    auto task_opt = replica.get_task(uuid_opt.value());
     if (!task_opt) {
       continue;
     }
-    auto task = std::move (task_opt.value ());
+    auto task = std::move(task_opt.value());
 
-    auto parent_uuid_opt = task.get_value ("parent");
+    auto parent_uuid_opt = task.get_value("parent");
     if (!parent_uuid_opt) {
       continue;
     }
-    auto parent_uuid = parent_uuid_opt.value ();
+    auto parent_uuid = parent_uuid_opt.value();
 
     if (parent_uuid == this_uuid) {
-      results.push_back (Task (std::move (task)));
+      results.push_back(Task(std::move(task)));
     }
   }
 
@@ -509,40 +482,30 @@ const std::vector <Task> TDB2::children (Task& parent)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string TDB2::uuid (int id)
-{
-  const tc::WorkingSet &ws = working_set ();
-  return ws.by_index ((size_t)id).value_or ("");
+std::string TDB2::uuid(int id) {
+  const tc::WorkingSet& ws = working_set();
+  return ws.by_index((size_t)id).value_or("");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int TDB2::id (const std::string& uuid)
-{
-  const tc::WorkingSet &ws = working_set ();
-  return (int)ws.by_uuid (uuid).value_or (0);
+int TDB2::id(const std::string& uuid) {
+  const tc::WorkingSet& ws = working_set();
+  return (int)ws.by_uuid(uuid).value_or(0);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int TDB2::num_local_changes ()
-{
-  return (int)replica.num_local_operations ();
-}
+int TDB2::num_local_changes() { return (int)replica.num_local_operations(); }
 
 ////////////////////////////////////////////////////////////////////////////////
-int TDB2::num_reverts_possible ()
-{
-  return (int)replica.num_undo_points ();
-}
+int TDB2::num_reverts_possible() { return (int)replica.num_undo_points(); }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::sync (tc::Server server, bool avoid_snapshots)
-{
+void TDB2::sync(tc::Server server, bool avoid_snapshots) {
   replica.sync(std::move(server), avoid_snapshots);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::dump ()
-{
+void TDB2::dump() {
   // TODO
 }
 
@@ -550,24 +513,16 @@ void TDB2::dump ()
 // For any task that has depenencies, follow the chain of dependencies until the
 // end.  Along the way, update the Task::is_blocked and Task::is_blocking data
 // cache.
-static void dependency_scan (std::vector<Task> &tasks)
-{
-  for (auto& left : tasks)
-  {
-    for (auto& dep : left.getDependencyUUIDs ())
-    {
-      for (auto& right : tasks)
-      {
-        if (right.get ("uuid") == dep)
-        {
+static void dependency_scan(std::vector<Task>& tasks) {
+  for (auto& left : tasks) {
+    for (auto& dep : left.getDependencyUUIDs()) {
+      for (auto& right : tasks) {
+        if (right.get("uuid") == dep) {
           // GC hasn't run yet, check both tasks for their current status
-          Task::status lstatus = left.getStatus ();
-          Task::status rstatus = right.getStatus ();
-          if (lstatus != Task::completed &&
-              lstatus != Task::deleted &&
-              rstatus != Task::completed &&
-              rstatus != Task::deleted)
-          {
+          Task::status lstatus = left.getStatus();
+          Task::status rstatus = right.getStatus();
+          if (lstatus != Task::completed && lstatus != Task::deleted &&
+              rstatus != Task::completed && rstatus != Task::deleted) {
             left.is_blocked = true;
             right.is_blocking = true;
           }
