@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,19 +27,20 @@
 #ifndef INCLUDED_CMDIMPORT
 #define INCLUDED_CMDIMPORT
 
-#include <string>
 #include <Command.h>
 #include <JSON.h>
 
-class CmdImport : public Command
-{
-public:
-  CmdImport ();
-  int execute (std::string&);
+#include <string>
 
-private:
-  int import (const std::string&);
-  void importSingleTask (json::object*);
+class CmdImport : public Command {
+ public:
+  CmdImport();
+  int execute(std::string&);
+
+ private:
+  std::unordered_map<std::string, unsigned int> uuid_occurrences;
+  int import(const std::string&);
+  void importSingleTask(json::object*);
 };
 
 #endif
