@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,21 +27,21 @@
 #ifndef INCLUDED_CMDMODIFY
 #define INCLUDED_CMDMODIFY
 
-#include <string>
 #include <Command.h>
 
-class CmdModify : public Command
-{
-public:
-  CmdModify ();
-  int execute (std::string&);
-  void checkConsistency (Task &before, Task &after);
-  int modifyAndUpdate (Task &before, Task &after,
-    std::map <std::string, std::string> *projectChanges = nullptr);
-  int modifyRecurrenceSiblings (Task &task,
-    std::map <std::string, std::string> *projectChanges = nullptr);
-  int modifyRecurrenceParent (Task &task,
-    std::map <std::string, std::string> *projectChanges = nullptr);
+#include <string>
+
+class CmdModify : public Command {
+ public:
+  CmdModify();
+  int execute(std::string &);
+  void checkConsistency(Task &before, Task &after);
+  int modifyAndUpdate(Task &before, Task &after,
+                      std::map<std::string, std::string> *projectChanges = nullptr);
+  int modifyRecurrenceSiblings(Task &task,
+                               std::map<std::string, std::string> *projectChanges = nullptr);
+  int modifyRecurrenceParent(Task &task,
+                             std::map<std::string, std::string> *projectChanges = nullptr);
 };
 
 #endif
