@@ -25,80 +25,96 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 #include <cmake.h>
+// cmake.h include header must come first
+
 #include <CmdSync.h>
-#include <sstream>
-#include <inttypes.h>
-#include <signal.h>
+#include <Color.h>
 #include <Context.h>
 #include <Filter.h>
-#include <Color.h>
-#include <shared.h>
 #include <format.h>
+#include <inttypes.h>
+#include <shared.h>
+#include <signal.h>
 #include <util.h>
+
+#include <sstream>
+
 #include "tc/Server.h"
 
 ////////////////////////////////////////////////////////////////////////////////
-CmdSync::CmdSync ()
-{
-  _keyword               = "synchronize";
-  _usage                 = "task          synchronize [initialize]";
-  _description           = "Synchronizes data with the Taskserver";
-  _read_only             = false;
-  _displays_id           = false;
-  _needs_gc              = false;
-  _uses_context          = false;
-  _accepts_filter        = false;
+CmdSync::CmdSync() {
+  _keyword = "synchronize";
+  _usage = "task          synchronize [initialize]";
+  _description = "Synchronizes data with the Taskserver";
+  _read_only = false;
+  _displays_id = false;
+  _needs_gc = false;
+  _uses_context = false;
+  _accepts_filter = false;
   _accepts_modifications = false;
   _accepts_miscellaneous = true;
-  _category              = Command::Category::migration;
+  _category = Command::Category::migration;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-int CmdSync::execute (std::string& output)
-{
+int CmdSync::execute(std::string& output) {
   int status = 0;
 
   tc::Server server;
   std::string server_ident;
 
   // If no server is set up, quit.
-  std::string origin = Context::getContext ().config.get ("sync.server.origin");
-  std::string server_dir = Context::getContext ().config.get ("sync.local.server_dir");
-  std::string gcp_credential_path = Context::getContext ().config.get ("sync.gcp.credential_path");
-  std::string gcp_bucket = Context::getContext ().config.get ("sync.gcp.bucket");
-  std::string encryption_secret = Context::getContext ().config.get ("sync.encryption_secret");
+  std::string origin = Context::getContext().config.get("sync.server.origin");
+  std::string url = Context::getContext().config.get("sync.server.url");
+  std::string server_dir = Context::getContext().config.get("sync.local.server_dir");
+  std::string gcp_credential_path = Context::getContext().config.get("sync.gcp.credential_path");
+  std::string gcp_bucket = Context::getContext().config.get("sync.gcp.bucket");
+  std::string encryption_secret = Context::getContext().config.get("sync.encryption_secret");
+
+  // sync.server.origin is a deprecated synonym for sync.server.url
+  std::string server_url = url == "" ? origin : url;
+
   if (server_dir != "") {
-    server = tc::Server::new_local (server_dir);
+    server = tc::Server::new_local(server_dir);
     server_ident = server_dir;
   } else if (gcp_bucket != "") {
     if (encryption_secret == "") {
-      throw std::string ("sync.encryption_secret is required");
+      throw std::string("sync.encryption_secret is required");
     }
-    server = tc::Server::new_gcp (gcp_bucket, gcp_credential_path, encryption_secret);
+    server = tc::Server::new_gcp(gcp_bucket, gcp_credential_path, encryption_secret);
     std::ostringstream os;
     os << "GCP bucket " << gcp_bucket;
     server_ident = os.str();
-  } else if (origin != "") {
-    std::string client_id = Context::getContext ().config.get ("sync.server.client_id");
+  } else if (server_url != "") {
+    std::string client_id = Context::getContext().config.get("sync.server.client_id");
     if (client_id == "" || encryption_secret == "") {
-      throw std::string ("sync.server.client_id and sync.encryption_secret are required");
+      throw std::string("sync.server.client_id and sync.encryption_secret are required");
     }
-    server = tc::Server::new_sync (origin, client_id, encryption_secret);
+    server = tc::Server::new_sync(server_url, client_id, encryption_secret);
     std::ostringstream os;
-    os << "Sync server at " << origin;
+    os << "Sync server at " << server_url;
     server_ident = os.str();
   } else {
-    throw std::string ("No sync.* settings are configured. See task-sync(5).");
+    throw std::string("No sync.* settings are configured. See task-sync(5).");
   }
 
   std::stringstream out;
-  if (Context::getContext ().verbose ("sync"))
-    out << format ("Syncing with {1}", server_ident)
-        << '\n';
+  if (origin != "") {
+    out << "sync.server.origin is deprecated. Use sync.server.url instead.\n";
+  }
 
-  Context::getContext ().tdb2.sync(std::move(server), false);
+  if (Context::getContext().verbose("sync")) {
+    out << format("Syncing with {1}", server_ident) << '\n';
+  }
 
-  output = out.str ();
+  Context& context = Context::getContext();
+  context.tdb2.sync(std::move(server), false);
+
+  if (context.config.getBoolean("purge.on-sync")) {
+    context.tdb2.expire_tasks();
+  }
+
+  output = out.str();
   return status;
 }
 
